using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel.common
{
    public class ObjectPositionSaver : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel"; // edited by VamTastic (original by Frief )
        public string pluginName = "ObjectPositionSaver";
        public string pluginVersion = "1";
        public string pluginDate = "14/01" + "/2023";
        public string pluginDescription = @"Used to open and close the doors";
        #endregion

        #region Vars    

        private Atom _thisAtom;
        private FreeControllerV3 _thisControl;

        private JSONStorableBool IsFirstLock;
        private JSONStorableStringChooser _positionSelectJSON;
        private UIDynamicPopup _positionDisplayPopup;
        private List<String> _positionObjChoices;
        private JSONStorableStringChooser _targetSelectJSON;
        private UIDynamicPopup _targetDisplayPopup;
        private List<String> _targetObjChoices;
        private List<String> _orificePersonChoices;
        private JSONStorableStringChooser _orificeJSON;
        private UIDynamicPopup _orificeDisplayPopup;
        private UIDynamicToggle _lock_toggle;

        private List<ObjectPositionData> _data_list;

        public Transform FirstGenRoot;
        public Transform SecondGenRoot;
        public Transform ThirdGenRoot;

        private Rigidbody _lNippleTrigger;
        private Rigidbody _rNippleTrigger;
        private Rigidbody _lipTrigger;
        private Rigidbody _mouthTrigger;
        private Rigidbody _labiaTrigger;
        private Rigidbody _vaginaTrigger;
        private Rigidbody _penisTipControl;

        protected JSONStorableFloat xPosition;
        protected JSONStorableFloat yPosition;
        protected JSONStorableFloat zPosition;
        protected JSONStorableFloat xRotation;
        protected JSONStorableFloat yRotation;
        protected JSONStorableFloat zRotation;

        protected JSONStorableFloat lockAngleX;
        protected JSONStorableFloat lockAngleY;
        protected JSONStorableFloat lockAngleZ;
        protected JSONStorableFloat lockPosX;
        protected JSONStorableFloat lockPosY;
        protected JSONStorableFloat lockPosZ;

        private Vector3 base_pos;
        private Quaternion base_quaternion;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute
        private bool is_person = false;



        #endregion

        public override void Init()
        {
            try
            {
				
                _thisAtom = containingAtom;
                _thisControl = _thisAtom.freeControllers.First(fc => fc.name == "control");

                _targetObjChoices = new List<string>();

                xPosition = new JSONStorableFloat("x position", 0f, OnChangeData, -100f, 100f);
                RegisterFloat(xPosition);
                yPosition = new JSONStorableFloat("y position", 0f, OnChangeData, -100f, 100f);
                RegisterFloat(yPosition);
                zPosition = new JSONStorableFloat("z position", 0f, OnChangeData, -100f, 100f);
                RegisterFloat(zPosition);

                xRotation = new JSONStorableFloat("x rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(xRotation);
                yRotation = new JSONStorableFloat("y rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(yRotation);
                zRotation = new JSONStorableFloat("z rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(zRotation);

                JSONStorableAction reset_action = new JSONStorableAction("Reset", TriggerSyncReset);
                RegisterAction(reset_action);

                UIDynamicButton btn1 = CreateButton("Default Save", false);
                btn1.button.onClick.AddListener(() =>
                {
                    SaveAtomData();
                });

                UIDynamicButton btn2 = CreateButton("Default Load", false);
                btn2.button.onClick.AddListener(() =>
                {
                    LoadAtomData();
                });

                IsFirstLock = new JSONStorableBool("Target Lock", false, OnIsFirstLock);
                RegisterBool(IsFirstLock);

                var btn4 = CreateButton("Target Update", false);

                btn4.button.onClick.AddListener(() =>
                {
                    OnTargetListUpdate();
                });

                _orificePersonChoices = new List<string>();
                _orificePersonChoices.Add("Mouth");
                _orificePersonChoices.Add("Vagina");
                _orificePersonChoices.Add("Anus");
                _orificePersonChoices.Add("rNipple");
                _orificePersonChoices.Add("lNipple");
                _orificePersonChoices.Add("penisTip");

                _targetSelectJSON = new JSONStorableStringChooser("Target Choice", null, null, "", OnFirstTargetSelect);
                _targetSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_targetSelectJSON);

                _targetDisplayPopup = CreateScrollablePopup(_targetSelectJSON, false);
                _targetDisplayPopup.popupPanelHeight = 700f;
                _targetDisplayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
                _targetDisplayPopup.label = "Target:";

                _targetSelectJSON.choices = _targetObjChoices;


                _orificeJSON = new JSONStorableStringChooser("OrificeChoice", null, "Vagina", "", OrificeSelect);
                _orificeJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_orificeJSON);

                _orificeJSON.choices = _orificePersonChoices;

                lockAngleX = new JSONStorableFloat("lock x rotation", 0f, OnChangeData, -180f, 180f);
                RegisterFloat(lockAngleX);
                CreateSlider(lockAngleX, true);
                lockAngleY = new JSONStorableFloat("lock y rotation", 0f, OnChangeData, -180f, 180f);
                RegisterFloat(lockAngleY);
                CreateSlider(lockAngleY, true);
                lockAngleZ = new JSONStorableFloat("lock z rotation", 0f, OnChangeData, -180f, 180f);
                RegisterFloat(lockAngleZ);
                CreateSlider(lockAngleZ, true);
                lockPosX = new JSONStorableFloat("lock x position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosX);
                CreateSlider(lockPosX, true);
                lockPosY = new JSONStorableFloat("lock y position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosY);
                CreateSlider(lockPosY, true);
                lockPosZ = new JSONStorableFloat("lock z position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosZ);
                CreateSlider(lockPosZ, true);


                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Object Position Saver</b></size></color>\n\n" +
                    "<color=#333>" +
                    "store the position of the target,\n" +
                    "or Track a specific target.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("DildoBooth ItemControl  : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

        }

        private bool InitEditor() {
            if (editorInitialized == true) return true;

            // I found my transforms, now the rest
            if (_thisAtom != null)
            {
                editorInitialized = true;

                if (xPosition.val == 0f && yPosition.val == 0f && zPosition.val == 0f) SaveAtomData();

                OnTargetListUpdate();

                if (_targetObjChoices.Count > 0)
                {
                    OnFirstTargetSelect(_targetSelectJSON.val);
                }
                //SuperController.LogMessage(">> " + _targetObjChoices.Count);

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless

                return true;
            }
            else
            {
                SuperController.LogError("DildoMovement : your is missing mandatory GlassBody.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

        }

        void FixedUpdate()
        {
            if (IsFirstLock.val)
            {
                if(!is_person)
                {
                    _thisControl.transform.position = FirstGenRoot.position;
                    Vector3 offsetPosition = new Vector3(lockPosX.val, lockPosY.val, lockPosZ.val);
                    _thisControl.transform.Translate(offsetPosition, _thisControl.transform);
                    _thisControl.transform.rotation = FirstGenRoot.rotation;
                    _thisControl.transform.Rotate(lockAngleX.val, lockAngleY.val, lockAngleZ.val);
                }
                else
                {
                    bool isl = SuperController.singleton.isLoading;
                    if (!isl)
                    {
                        if (_labiaTrigger != null)
                        {
                            //SuperController.LogMessage(">> " + _orificeJSON.val);
                            if (_orificeJSON.val == "Mouth")
                            {
                                _thisControl.transform.position = _lipTrigger.transform.position;
                                _thisControl.transform.rotation = Quaternion.LookRotation(_mouthTrigger.transform.position - _lipTrigger.transform.position, _lipTrigger.transform.up);
                            }
                            if (_orificeJSON.val == "rNipple")
                            {
                                _thisControl.transform.position = _rNippleTrigger.transform.position;
                                _thisControl.transform.rotation = _rNippleTrigger.transform.rotation;
                            }
                            if (_orificeJSON.val == "lNipple")
                            {
                                _thisControl.transform.position = _lNippleTrigger.transform.position;
                                _thisControl.transform.rotation = _lNippleTrigger.transform.rotation;
                            }
                            if (_orificeJSON.val == "Vagina")
                            {
                                _thisControl.transform.position = _labiaTrigger.transform.position;
                                _thisControl.transform.rotation = Quaternion.LookRotation(_vaginaTrigger.transform.position - _labiaTrigger.transform.position, _labiaTrigger.transform.up);
                            }
                            if (_orificeJSON.val == "penisTip")
                            {
                                _thisControl.transform.position = _penisTipControl.transform.position;
                                _thisControl.transform.rotation = _penisTipControl.transform.rotation;
                            }
                            if (_orificeJSON.val == "Anus")
                            {
                                Transform anusTransform = _thisControl.transform;

                                _thisControl.transform.position = _labiaTrigger.transform.position;
                                _thisControl.transform.rotation = Quaternion.LookRotation(_vaginaTrigger.transform.position - _labiaTrigger.transform.position, _labiaTrigger.transform.up);

                                Vector3 anusOffsetPosition = new Vector3(0.0f, -0.025f, 0.008f);
                                anusTransform.Translate(anusOffsetPosition, anusTransform);

                                _thisControl.transform.position = anusTransform.position;
                                _thisControl.transform.rotation = anusTransform.rotation;
                                //_thisControl.transform.localRotation = Quaternion.identity;
                                //anusTransform.Translate(anusOffsetPosition, anusTransform);

                                _thisControl.transform.rotation = Quaternion.LookRotation(_vaginaTrigger.transform.position - anusTransform.position, anusTransform.up);

                                //anusTransform.position = tempPosition;
                                //anusTransform.rotation = tempRotation;
                            }

                            Vector3 offsetPosition = new Vector3(lockPosX.val, lockPosY.val, lockPosZ.val);
                            _thisControl.transform.Translate(offsetPosition, _thisControl.transform);

                            _thisControl.transform.Rotate(lockAngleX.val, lockAngleY.val, lockAngleZ.val);
                            //SuperController.LogMessage(">> " + lockAngleX.val + " - " + _thisControl.transform.localEulerAngles);

                        }
                    }
                }
            }
        }

        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        // This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
        private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				if( child.name == searchName ) {
					
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}

        private void OnIsFirstLock(JSONStorableBool jb)
        {
            if(!jb.val) LoadAtomData();
        }

        private void OnFirstTargetSelect(string target_name)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(target_name);
            //SuperController.LogMessage("OnFirstTargetSelect >> " + target_name + " - " + atom.type);

            FirstGenRoot = atom.freeControllers.First(fc => fc.name == "control").transform;

            if (atom.type == "Person")
            {
                _lipTrigger = atom.rigidbodies.First(rb => rb.name == "LipTrigger");
                _mouthTrigger = atom.rigidbodies.First(rb => rb.name == "MouthTrigger");
                _labiaTrigger = atom.rigidbodies.First(rb => rb.name == "LabiaTrigger");
                _vaginaTrigger = atom.rigidbodies.First(rb => rb.name == "VaginaTrigger");
                _rNippleTrigger = atom.rigidbodies.First(rb => rb.name == "rNippleTrigger");
                _lNippleTrigger = atom.rigidbodies.First(rb => rb.name == "lNippleTrigger");
                _penisTipControl = atom.rigidbodies.First(rb => rb.name == "penisTipControl");

                is_person = true;
                ShowOrificeDisplayPopup(true);
                
            }
            else
            {
                _lipTrigger = null;
                _mouthTrigger = null;
                _labiaTrigger = null;
                _vaginaTrigger = null;
                _rNippleTrigger = null;
                _lNippleTrigger = null;
                _penisTipControl = null;

                is_person = false;
                ShowOrificeDisplayPopup(false);
                
            }
                

        }

        protected void OrificeSelect(string target_name)
        {
            //SuperController.LogMessage(">> " + target_name + " - " + _orificeJSON.val);
            
        }

        protected void SyncOrificeChoices()
        {
            //SuperController.LogMessage(">> SyncOrificeChoices ");
            //UpdateOrificeChoices();
        }

        private void OnTargetListUpdate()
        {
            List<string> uids = SuperController.singleton.GetAtomUIDs();

            _targetObjChoices = new List<string>();

            foreach (string uid in uids)
            {
                //SuperController.LogMessage(">> OnTargetListUpdate " + _thisAtom.name + " - " + uid);
                //Atom atom = SuperController.singleton.GetAtomByUid(uid);
                if(_thisAtom.name != uid) _targetObjChoices.Add(uid);
            }

            _targetSelectJSON.choices = _targetObjChoices;
        }

        private void SaveAtomData()
        {
            Vector3 base_pos = _thisControl.transform.position;
            Vector3 base_angle = _thisControl.transform.eulerAngles;

            xPosition.val = base_pos.x;
            yPosition.val = base_pos.y;
            zPosition.val = base_pos.z;

            xRotation.val = base_angle.x;
            yRotation.val = base_angle.y;
            zRotation.val = base_angle.z;

        }

        private void LoadAtomData()
        {
            _thisControl.transform.position = new Vector3(xPosition.val, yPosition.val, zPosition.val);
            _thisControl.transform.rotation = Quaternion.Euler(new Vector3(xRotation.val, yRotation.val, zRotation.val));

        }

        void OnChangeData(float value)
        {
            
        }

        private void TriggerSyncReset()
        {
            IsFirstLock.val = false;
            LoadAtomData();
        }

        private void ShowOrificeDisplayPopup(bool is_show)
        {
            //SuperController.LogMessage("ShowOrificeDisplayPopup >> " + is_show);
            if (_lock_toggle != null) RemoveToggle(_lock_toggle);
            if (is_show)
            {
                if(_orificeDisplayPopup == null) _orificeDisplayPopup = CreateScrollablePopup(_orificeJSON, false);
                _orificeDisplayPopup.popupPanelHeight = 600f;
                _orificeDisplayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
                _orificeDisplayPopup.label = "Target Orifice:";
            }
            else
            {
                if(_orificeDisplayPopup != null) RemovePopup(_orificeDisplayPopup);
            }

            //SuperController.LogMessage("ShowOrificeDisplayPopup >> " + is_show);
            _lock_toggle = CreateToggle(IsFirstLock, false);
            _lock_toggle.label = "First Lock On Off";
        }

    }

    public class ObjectPositionData
    {
        private string _name;
        private Vector3 _position;
        private Vector3 _rotation;

        public string name
        {
            get { return _name; }
        }

        public Vector3 position
        {
            get { return _position; }
        }

        public Vector3 rotation
        {
            get { return _rotation; }
        }

        public void UpdatePosition(Vector3 pos)
        {
            _position = pos;
        }

        public void UpdateRotation(Vector3 rot)
        {
            _rotation = rot;
        }

    }
	
}